import pandas as pd
import os
import datetime as dt
import json

def repo_id_from_upstream(vcs_link):
    if 'github' in vcs_link:
        repo_id = vcs_link[len('https://github.com/'):]
    else:
        parts = vcs_link.split('/')
        repo_id = '/'.join(parts[3:])
    und_repo_id = '_'.join(repo_id.split("/"))
    return und_repo_id

def get_readme_list(csv_path):
    #main_df = pd.read_csv('final_data/deb_full_data.csv')
    existing_list = pd.read_csv(csv_path)
    existing_list['repo_id'] = existing_list['upstream_vcs_link'].apply(repo_id_from_upstream)
    present_data = existing_list[['upstream_vcs_link', 'repo_id']]
    present_data.to_csv('012925_readme_manifest.csv', index=False)


def get_contributing_list(csv_path):
    existing_list = pd.read_csv(csv_path)
    existing_list['repo_id'] = existing_list['upstream_vcs_link'].apply(repo_id_from_upstream)
    present_data = existing_list[['upstream_vcs_link', 'repo_id']]
    present_data.to_csv('012925_contributing_manifest.csv', index=False)


def check_collected_files(dirpath):
    if "contributing" in dirpath:
        manifest = pd.read_csv('012925_contributing_manifest.csv')
    else:
        manifest = pd.read_csv('012925_readme_manifest.csv')

    csv_files = [f for f in os.listdir(dirpath) if f.endswith('_commits.csv')]
    file_ids = {f[1:-12] if f.startswith('_') else f[:-12] for f in csv_files}

    manifest_ids = set(manifest["repo_id"])

    matched_repo_ids = file_ids & manifest_ids
    unmatched_repo_ids = file_ids - matched_repo_ids
    unmatched_files = [f for f in csv_files if (f[1:-12] if f.startswith('_') else f[:-12]) in unmatched_repo_ids]

    return unmatched_files, matched_repo_ids

def cp_existing_docs(matched_repo_ids, is_readme):
    if not is_readme:
        manifest = pd.read_csv('012925_contributing_manifest.csv')
    else:
        manifest = pd.read_csv('012925_readme_manifest.csv')
    # match the ids to the upstream vcs links 
    # match those to the files in "/data/users/mgaughan/time_specific_files/{doctype}3"
    # copy matched files to "/data/users/mgaughan/kkex/012825_cam_revision_main/first_version_documents/{doctype}/"

def document_in_diffs(diffs, doc_name):
        #return any(d['parent_filepath'] == doc_name or d['child_filepath'] == doc_name for d in diffs)
        if f"'parent_filepath': '{doc_name}'" in diffs:
            return True
        if f"'child_filepath': '{doc_name}'" in diffs:
            return True
        return False

def get_doc_publication(dirpath, file, doc_name):
    unmatched_df = pd.read_csv(dirpath + file)
    # because we only care about the date, not going to un-flatten the timezones
    unmatched_df['commit_date'] = pd.to_datetime(unmatched_df['commit_date'], utc=True)
    unmatched_df = unmatched_df.sort_values(by='commit_date', ascending=False)
    unmatched_df['diff_contains_document'] = unmatched_df['diff_info'].apply(lambda diffs: document_in_diffs(diffs, doc_name))
    first_instance = unmatched_df[unmatched_df['diff_contains_document']].head(1)
    #first_instance = unmatched_df.loc["'parent_filepath': 'README'" in unmatched_df['diff_info']].head(1)
    return first_instance

def get_doc_pub_for_all( is_readme):
    if is_readme:
        dirpath = "/data/users/mgaughan/kkex/012825_cam_revision_main/main_commit_data/readme/"
        doc_name = "README"
    else: 
        dirpath = "/data/users/mgaughan/kkex/012825_cam_revision_main/main_commit_data/contributing/"
        doc_name = "CONTRIBUTING"
    csv_files = [f for f in os.listdir(dirpath) if f.endswith('_commits.csv')]
    first_instances = []
    for file in csv_files:
        first_instance = get_doc_publication(dirpath, file, doc_name)
        if file.startswith('_'):
            repo_id = file[1:-12]
        else:
            repo_id = file[:-12]
        first_instance['repo_id'] = repo_id
        #   find the upstream vcs from the full_data list 
        if not first_instance.empty:
            first_instances.append(first_instance)
    #save the 
    #first_instances_df = pd.concat(first_instances, ignore_index=True)

def list_for_doc_collection(unmatched_files, is_readme):
    if is_readme:
        dirpath = "/data/users/mgaughan/kkex/012825_cam_revision_main/main_commit_data/readme/"
        doc_name = "README"
    else: 
        dirpath = "/data/users/mgaughan/kkex/012825_cam_revision_main/main_commit_data/contributing/"
        doc_name = "CONTRIBUTING"
    full_data = pd.read_csv('for_batching/deb_full_data.csv')
    first_instances = []
    for file in unmatched_files:
        first_instance = get_doc_publication(dirpath, file, doc_name)
        if file.startswith('_'):
            repo_id = file[1:-12]
        else:
            repo_id = file[:-12]
        first_instance['repo_id'] = repo_id
        #   find the upstream vcs from the full_data list 
        if not first_instance.empty:
            first_instances.append(first_instance)
        break
    #first_instances_df = pd.concat(first_instances, ignore_index=True)

if __name__ == "__main__":
    #get_contributing_licsv_files = [f for f in os.listdir(dirpath) if f.endswith('_commits.csv')]
    is_readme = True
    unmatched_files, matched_repo_ids = check_collected_files("/data/users/mgaughan/kkex/012825_cam_revision_main/main_commit_data/contributing")
    #list_for_doc_collection(unmatched_files, is_readme)
    print(len(unmatched_files))
    print(len(matched_repo_ids))